@@ -14,14 +14,12 @@
 # limitations under the License.
 
 import time
-
-from email.utils import parsedate_tz, mktime_tz
+from email.utils import mktime_tz, parsedate_tz
 
 __all__ = ["BaseHTTPError", "RateLimitReachedError", "exception_from_message"]
 
 
 class BaseHTTPError(Exception):
-
     """
     The base exception class for all HTTP related exceptions.
     """
@@ -32,7 +30,7 @@ class BaseHTTPError(Exception):
         self.headers = headers
         # preserve old exception behavior for tests that
         # look for e.args[0]
-        super(BaseHTTPError, self).__init__(message)
+        super().__init__(message)
 
     def __str__(self):
         return self.message
@@ -48,7 +46,7 @@ class RateLimitReachedError(BaseHTTPError):
 
     def __init__(self, *args, **kwargs):
         headers = kwargs.pop("headers", None)
-        super(RateLimitReachedError, self).__init__(self.code, self.message, headers)
+        super().__init__(self.code, self.message, headers)
         if self.headers is not None:
             self.retry_after = float(self.headers.get("retry-after", 0))
         else:
@@ -56,7 +54,7 @@ class RateLimitReachedError(BaseHTTPError):
 
 
 _error_classes = [RateLimitReachedError]
-_code_map = dict((c.code, c) for c in _error_classes)
+_code_map = {c.code: c for c in _error_classes}
 
 
 def exception_from_message(code, message, headers=None):
